/* client.cc: XMMS2 client implementation.
 *
 * Copyright (c) 2006 Jonne Lehtinen <jms@modeemi.fi>
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the Do What The F*ck You Want To
 *  Public License, Version 2, as published by Sam Hocevar. See
 *  COPYING file for more details.
 */

#include <xmmsclient/xmmsclient++-glib.h>

#include <cstdlib>
#include <iostream>

#include <glibmm/ustring.h>
#include <gtkmm/main.h>

#include "client.hh"
#include "main_window.hh"

gx2osd::client::client()
    : client_( "gx2osd" ),
      win_( get_window() ) {

    const char* path = std::getenv( "XMMS_PATH" );
    client_.connect( path ? path : "" );
    win_.set_status_no_show( client_.playback.getStatus() );

    client_.setMainloop( new Xmms::GMainloop( client_.getConnection() ) );
    client_.playback.currentID( Xmms::bind( &client::song_changed, this ) );

    client_.playback.broadcastCurrentID( 
        Xmms::bind( &client::song_changed, this ) );
    client_.playback.broadcastStatus( Xmms::bind( &main_window::set_status, &win_ ) );
    client_.setDisconnectCallback( boost::bind( &client::dc_handler, this ) );

}

gx2osd::client::~client() {

}

bool
gx2osd::client::song_changed( const unsigned int& id ) {
    client_.medialib.getInfo( id, Xmms::bind( &client::info_get, this ) );
    return true;
}

bool 
gx2osd::client::error_handler( const std::string& error,
                               const Glib::ustring& artist,
                               const Glib::ustring& title,
                               const Glib::ustring& album ) {

    std::cerr << "Error: " << error << std::endl;
    win_.set_text_and_show( artist, title, album );
    return false;

}

bool
gx2osd::client::info_get( const Xmms::PropDict& info ) {

    Glib::ustring artist;
    Glib::ustring title;
    Glib::ustring album;

    if( info.contains( "artist" ) ) {
        artist = info.get< std::string >( "artist" );
    }
    if( info.contains( "title" ) ) {
        title = info.get< std::string >( "title" );
    }
    if( info.contains( "album" ) ) {
        album = info.get< std::string >( "album" );
    }
    if( info.contains( "picture_front" ) ||
        info.contains( "picture_front_thumbnail" ) ) {

        std::string hash;

        if( info.contains( "picture_front_thumbnail" ) ) {
            hash = info.get< std::string >( "picture_front_thumbnail" );
        }
        else {
            hash = info.get< std::string >( "picture_front" );
        }

        client_.bindata.retrieve( hash, 
                                  boost::bind( &client::cover_get, this,
                                               _1, artist, title, album ),
                                  boost::bind( &client::error_handler, this,
                                               _1, artist, title, album ) );

    }
    else {

        win_.set_text_and_show( artist, title, album );

    }

    return true;
}

bool
gx2osd::client::cover_get( const Xmms::bin& cover,
                           const Glib::ustring& artist,
                           const Glib::ustring& title,
                           const Glib::ustring& album ) {

    win_.set_cover( cover, artist, title, album );
    return false;

}

void
gx2osd::client::dc_handler() {
    std::clog << "Disconnected..." << std::endl;
    Gtk::Main::quit();
}
