/* skin.hh: skin format header.
 *
 * Copyright (c) 2006 Jonne Lehtinen <jms@modeemi.fi>
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the Do What The F*ck You Want To
 *  Public License, Version 2, as published by Sam Hocevar. See
 *  COPYING file for more details.
 */

#ifndef GX2OSD_SKIN_HH
#define GX2OSD_SKIN_HH

#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/dom/DOMElement.hpp>

#include "boost/noncopyable.hpp"
#include "boost/shared_ptr.hpp"
#include "boost/filesystem/path.hpp"

#include <string>

#include "image.hh"
#include "label.hh"

namespace fs = boost::filesystem;

namespace gx2osd {

    class skin : private boost::noncopyable {

        public:
            skin( const std::string& name );
            ~skin();

            image& background();
            image& no_cover_image();
            image& paused_image();
            image& stopped_image();
            image& playing_image();

            const Position& get_status_position() const;

            const label& get_artist() const;
            const label& get_album() const;
            const label& get_songname() const;

            bool has_album() const;

        private:

            typedef boost::shared_ptr< XMLCh > xmlch;

            const std::string name_;
            fs::path skin_path_;

            image background_;
            image no_cover_image_;
            image paused_image_;
            image stopped_image_;
            image playing_image_;

            label artist_;
            label album_;
            label songname_;

            bool has_album_;

            Position status_position_;

            void load_image( image& i, xmlch tag_name,
                             xercesc::DOMDocument*& doc );

            bool load_label( label& l, xmlch tag_name,
                             xercesc::DOMDocument*& doc );

            void parse_position( xercesc::DOMElement*& element,
                                 unsigned int& x, unsigned int& y );

            xercesc::DOMElement* find_element( xmlch tag_name,
                                               xercesc::DOMDocument*& doc );

    };

}

#endif // GX2OSD_SKIN_HH
